'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _lodash = require('lodash');

var _highlightEs = require('highlight-es');

var _highlightEs2 = _interopRequireDefault(_highlightEs);

var _defaultOptions = require('./default-options');

var _defaultOptions2 = _interopRequireDefault(_defaultOptions);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _utils = require('../utils');

var _cliTable = require('cli-table');

var _cliTable2 = _interopRequireDefault(_cliTable);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/// @name Console
/// @page api
/// @description This is used to output data to the console
var Console = function (_Base) {
  (0, _inherits3.default)(Console, _Base);

  ///# @name constructor
  ///# @arg {object} options - The base options
  ///# @arg {object} output_options - The output options
  function Console() {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var output_options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck3.default)(this, Console);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Console.__proto__ || (0, _getPrototypeOf2.default)(Console)).call(this, options));

    _this.output_options = (0, _lodash.extend)({
      highlight: true
    }, _defaultOptions2.default, output_options);

    _this.prepared = false;
    return _this;
  }

  ///# @name prepare
  ///# @description
  ///# This is used to prepare the saving functionality that is determined by the
  ///# options that were passed to the constructor.
  ///# It sets a variable of `this.preparing` that ultimately calls `this.setup` that returns a promise.
  ///# This way when you go to save data it, that function will know if the setup is complete or not and
  ///# wait for it to be done before it starts saving data.
  ///# @returns {promise} - The setup function that was called
  ///# @async


  (0, _createClass3.default)(Console, [{
    key: 'prepare',
    value: function prepare() {
      this.preparing = true;
      this.preparing = this.setup();
      return this.preparing;
    }

    ///# @name setup
    ///# @description
    ///# This is used to setup the saving function that will be used.

  }, {
    key: 'setup',
    value: function () {
      var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
        var _this2 = this;

        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!(this.preparing == null)) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', this.prepare());

              case 2:
                return _context.abrupt('return', new _promise2.default(function (resolve) {
                  // theres noting to setup for the Console output
                  // this is just here so that all the Outputters are setup the same
                  process.nextTick(function () {
                    _this2.prepared = true;
                    resolve();
                  });
                }));

              case 3:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function setup() {
        return _ref.apply(this, arguments);
      }

      return setup;
    }()

    ///# @name output
    ///# @description
    ///# This is used to output the data that's passed to it
    ///# @arg {string} id - The id to use for this data
    ///# @arg {object, array, string} data - The data that you want to be saved
    ///# @async

  }, {
    key: 'output',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(id, data) {
        var table;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!(this.prepared !== true)) {
                  _context2.next = 4;
                  break;
                }

                if (this.preparing == null) {
                  this.prepare();
                }
                _context2.next = 4;
                return this.preparing;

              case 4:
                if (!this.output_options.highlight) {
                  _context2.next = 17;
                  break;
                }

                if (!(this.output_options.format === 'csv')) {
                  _context2.next = 15;
                  break;
                }

                table = new _cliTable2.default();
                _context2.next = 9;
                return _utils.parsers.csv.parse(data);

              case 9:
                data = _context2.sent;


                table.push((0, _lodash.keys)(data[0]).map(function (key) {
                  return _chalk2.default.bold(key);
                }));
                data.forEach(function (obj) {
                  return table.push((0, _lodash.values)(obj));
                });
                data = table;
                _context2.next = 16;
                break;

              case 15:
                data = (0, _highlightEs2.default)(data);

              case 16:

                data += '';

              case 17:
                return _context2.abrupt('return', data);

              case 18:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function output(_x3, _x4) {
        return _ref2.apply(this, arguments);
      }

      return output;
    }()
  }]);
  return Console;
}(_base2.default);

exports.default = Console;
//# sourceMappingURL=console.js.map